﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Problems.TradeRules {
  [StorableClass]
  public abstract class TradeRulesAnalysisSingleObjectiveEvaluator<T> : TradeRulesAnalysisEvaluator<T>, ISymbolicDataAnalysisSingleObjectiveEvaluator<T>
   where T : class, IDataAnalysisProblemData {
    private const string QualityParameterName = "Quality";
    #region parameter properties
    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters[QualityParameterName]; }
    }
    #endregion
    #region properties
    public abstract bool Maximization { get; }
    #endregion
    [StorableConstructor]
    protected TradeRulesAnalysisSingleObjectiveEvaluator(bool deserializing) : base(deserializing) { }
    protected TradeRulesAnalysisSingleObjectiveEvaluator(TradeRulesAnalysisSingleObjectiveEvaluator<T> original, Cloner cloner)
      : base(original, cloner) {
    }

    protected TradeRulesAnalysisSingleObjectiveEvaluator()
      : base() {
      Parameters.Add(new LookupParameter<DoubleValue>(QualityParameterName, "The quality of the evaluated symbolic data analysis solution."));
    }

    public abstract double Evaluate(IExecutionContext context, ISymbolicExpressionTree tree, T problemData, IEnumerable<int> rows);
  }
}
